; OCURVECHANGE__DEFINE.PRO
;
; Object widget program that illustrates how to implement modal and non-modal
; dialog widget programs.
;
; Written by R.M. Dimeo (12/25/02)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro ocurveCleanup,tlb
widget_control,tlb,get_uvalue = self
obj_destroy,self
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro ocurveChange::cleanup
wdelete,self.winPix
ptr_free,self.dataPtr
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro ocurveChange::quit, event = event
widget_control,self.tlb,/destroy
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro ocurveChange::nonModalDialog,event = event
nmoCurve,group_leader = self.tlb
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro ocurveChange::modalDialog,event = event
newData = mCurveChange(parent = self.tlb,(*self.dataPtr)[*,0])
; Get here if the DONE button was pressed.
self->setProperty,data = newData,/draw
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro ocurveChange::draw,event = event
data = *self.dataPtr
x = data[*,0]
y = data[*,1]
wset,self.winPix
plot,x,y,psym = 0,xrange = [min(x),max(x)],/xsty
wset,self.winVis
device,copy = [0,0,!d.x_size,!d.y_size,0,0,self.winPix]
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro ocurveChange::setProperty,data = data,draw = draw
if n_elements(data) ne 0 then *self.dataPtr = data
if keyword_set(draw) then self->draw
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro ocurveChange::calcFun,funcName
(*self.dataPtr)[*,1] = call_function(funcName,(*self.dataPtr)[*,0])
self->draw
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
PRO ocurveEvents, event
; This is the only event handler necessary for the program.  It makes a
; generic method call for an object, the information being provided
; by the UVALUE of the calling widgets.
widget_control, event.id, get_uvalue=cmd
call_method, cmd.method,cmd.object,event = event
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro ocurveChange::createWidgets
; Widget definition module
thisTitle = 'Object widget implementation ('+strtrim(string(self.count),2)+')'
if self.group_leader eq 0L then begin
  self.tlb = widget_base(/col,/tlb_frame_attr,mbar = bar,title = thisTitle)
endif else begin
  self.tlb = widget_base(group_leader = self.group_leader,/col, $
    /tlb_frame_attr,mbar = bar,title = thisTitle)
endelse
filemenu = widget_button(bar,value = 'FILE',/menu)
void = widget_button(filemenu,value = 'QUIT', $
	uvalue = {object:self,method:'quit'})

xsize = 400 & ysize = 400
win = widget_draw(self.tlb,xsize = xsize,ysize = ysize)
rowBase = widget_base(self.tlb,/row,/align_center)

void = widget_button(rowBase,value = 'MODAL DIALOG', $
	uvalue = {object:self,method:'modalDialog'})
void = widget_button(rowBase,value = 'NON-MODAL DIALOG', $
	uvalue = {object:self,method:'nonModalDialog'})
widget_control,self.tlb,/realize
widget_control,win,get_value = winVis
self.winVis = winVis
window,/free,/pixmap,xsize = xsize,ysize = ysize
self.winPix = !d.window
widget_control,self.tlb,set_uvalue = self
xmanager,'oc',self.tlb,$
         event_handler = 'ocurveEvents',$
         cleanup = 'ocurveCleanup',$
         /no_block
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function ocurveChange::init,group_leader = group_leader,count = count
device,decomposed = 0
loadct,0,/silent
if n_elements(group_leader) eq 0 then group_leader = 0L
self.group_leader = group_leader
if n_elements(count) eq 0 then count = 0
self.count = count
; create a sin function to display
n = 50
x = (2.0*!pi/(n-1.))*findgen(n)
y = sin(x)
data = [[x],[y]]
self.dataPtr = ptr_new(data,/no_copy)
self->createWidgets
self->draw
return,1
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro ocurveChange__define
define = 	{	ocurveChange,			$
				group_leader:0L,		$
				count:0,				$
				winVis:0L,				$
				winPix:0L,				$
				tlb:0L,					$
				dataPtr:ptr_new()		$
			}

end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro curveChange
o = obj_new('ocurveChange')
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
