; DETACHEDWIDGET.PRO
;
; This simple widget program shows how you can use a "HIDE/SHOW" feature
; to selectively display a detached widget control base.  This program
; uses MAPPING to toggle hiding and showing the base.  Some notable features
; are that it calls XMANAGER twice to register two bases with the local windows
; manager and it also sets the UVALUE of each base to the state pointer.
;
; Written by R.M. Dimeo (11/26/02)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro dwCleanup,tlb
widget_control,tlb,get_uvalue = pState
ptr_free,pState
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro dwEvent,event
uname = widget_info(event.id,/uname)
case uname of
'QUIT':	$
  begin
    widget_control,event.top,/destroy
  end
'SHOW':	$
  begin
    widget_control,event.top,get_uvalue = pState
    widget_control,(*pState).ctrlBase,/map
    widget_control,event.id,set_uname = 'HIDE'
  end
'HIDE':	$
  begin
    widget_control,event.top,get_uvalue = pState
    widget_control,(*pState).ctrlBase,map = 0
    widget_control,event.id,set_uname = 'SHOW'
  end
; The following unames correspond to events coming from
; the control base.
'APPLE': $
  begin
    ; Note that here, EVENT.TOP is CTRLBASE itself.
	widget_control,event.top,get_uvalue = pState
    widget_control,(*pState).text,set_value = 'APPLE'
  end
'ORANGE': $
  begin
	widget_control,event.top,get_uvalue = pState
    widget_control,(*pState).text,set_value = 'ORANGE'
  end
else:
endcase

end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro detachedWidget
; Widget definition module
tlb = widget_base(/col,title = 'Detached widget example')
text = widget_text(tlb,value = '',xsize = 35)
hide = widget_button(tlb,value = 'HIDE/SHOW',uname = 'SHOW')
quit = widget_button(tlb,value = 'QUIT',uname = 'QUIT')
; Create the CTRLBASE, do not map it, but put the value of the TLB into the
; UVALUE of CTRLBASE.
ctrlBase = widget_base(group_leader = tlb,/col,title = 'Controls',map = 0,xsize = 100)
apple = widget_button(ctrlBase,value = 'APPLE',uname = 'APPLE')
orange = widget_button(ctrlBase,value = 'ORANGE',uname = 'ORANGE')

; We must realize BOTH bases!
widget_control,tlb,/realize
widget_control,ctrlBase,/realize

state = {ctrlBase:ctrlBase,text:text}
pState = ptr_new(state,/no_copy)

; Set the UVALUES of both tlb and ctrlBase to pState!
widget_control,tlb,set_uvalue = pState
widget_control,(*pState).ctrlBase,set_uvalue = pState
; Now we must register BOTH bases with the local windows manager using xmanager.
xmanager,'dw',tlb,cleanup = 'dwCleanup',/no_block, $
  event_handler = 'dwEvent'
xmanager,'dw',(*pState).ctrlBase,event_handler = 'dwEvent'
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;